% Monte Carlo simulation of 1-D diffusion from a point source
% A total of N particles is placed at a a single position at t=0.
% Each particle jumps with a probability W to the left or right with equal
% probabilities. Each partilce remains in its poistion over one timestep
% with the probabilty 1-W.

clear;

% Discretization
X=30;     % X:    number of points on x axis in the positive and negative directions
x=-X:X;   % x:    equally spaced points along x axis (vector of length 2X+1)
T=50;     % T:    nubmer of time steps/jumps  

% initialization 
N=10000;               % total number of particles
n=zeros(1,2*X+1);     % vector containing the particle numbers for each x
%n(X)=N/2;            
n(X+1)=N;             % the point source of N particles is placed at the X+1 node
%n(X+2)=N/2;          
m=zeros(1,2*X+1);     % auxiliary vector to store particle numbers after each time step

% loope over time, positions and particles
for i=1:T                 % loop over all time steps
    for j=2:(2*X)         % loop over all positions in x
        if n(j)>0
            for l=1:n(j)  % loop over all n(j) particles at position j
                r=rand;   % create a random number 0<rand<1
                if r < .5
                    m(j-1)=m(j-1)+1;                    
                else
                    m(j+1)=m(j+1)+1;
                end;
            end;
        end;
    end;
    m(1)=0;
    m(end)=0;
    n=m;
    m=zeros(1,2*X+1);
    plot(x,n,'.'), axis([-X X 0 2*N/10]); pause(0.1)
end;
% Analysis of simulation results; comparison with the Gaussian distribution

D=0.5;
yf=2*N*exp(-x.^2/(4*D*T))./sqrt(4*pi*D*T);
figure;
plot(x,n,'.',x,yf); axis([-X X 0 2*N/10]); legend('rand','Gaussian');